==> sysprof_3.38.1.orig.tar.xz: sysprof-3.38.1/src/libsysprof-capture/sysprof-platform.c <==
/* sysprof-platform.c
 *
 * Copyright 2016-2019 Christian Hergert <chergert@redhat.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * Subject to the terms and conditions of this license, each copyright holder
 * and contributor hereby grants to those receiving rights under this license
 * a perpetual, worldwide, non-exclusive, no-charge, royalty-free,
 * irrevocable (except for failure to satisfy the conditions of this license)
 * patent license to make, have made, use, offer to sell, sell, import, and
 * otherwise transfer this software, where such license applies only to those
 * patent claims, already acquired or hereafter acquired, licensable by such
 * copyright holder or contributor that are necessarily infringed by:
 *
 * (a) their Contribution(s) (the licensed copyrights of copyright holders
 *     and non-copyrightable additions of contributors, in source or binary
 *     form) alone; or
 *
 * (b) combination of their Contribution(s) with the work of authorship to
 *     which such Contribution(s) was added by such copyright holder or
 *     contributor, if, at the time the Contribution is added, such addition
 *     causes such combination to be necessarily infringed. The patent license
 *     shall not apply to any other combinations which include the
 *     Contribution.
 *
 * Except as expressly stated above, no rights or licenses from any copyright
 * holder or contributor is granted under this license, whether expressly, by
 * implication, estoppel or otherwise.
 *
 * DISCLAIMER
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * SPDX-License-Identifier: BSD-2-Clause-Patent
 */

#include "config.h"

#include <stdlib.h>
#include <sys/syscall.h>
#include <unistd.h>

#include "sysprof-capture-util-private.h"
#include "sysprof-platform.h"

#ifndef __NR_memfd_create
static const char *
get_tmpdir (void)
{
  const char *tmpdir = NULL;

  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = getenv ("TMPDIR");

#ifdef P_tmpdir
  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = P_tmpdir;
#endif

  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = "/tmp";

  return tmpdir;
}
#endif  /* !__NR_memfd_create */

/**
 * sysprof_memfd_create:
 * @name: (nullable): A descriptive name for the memfd or %NULL
 *
 * Creates a new memfd using the memfd_create syscall if available.
 * Otherwise, a tmpfile is used. Currently, no attempt is made to
 * ensure the tmpfile is on a tmpfs backed mount.
 *
 * Returns: An fd if successful; otherwise -1 and errno is set.
 */
int
sysprof_memfd_create (const char *name)
{
#ifdef __NR_memfd_create
  if (name == NULL)
    name = "[sysprof]";
  return syscall (__NR_memfd_create, name, 0);
#else
  int fd;
  const char *tmpdir;
  char *template = NULL;
  size_t template_len = 0;

  /*
   * TODO: It would be nice to ensure tmpfs
   *
   * It is not strictly required that the preferred temporary directory
   * will be mounted as tmpfs. We should look through the mounts and ensure
   * that the tmpfile we open is on tmpfs so that we get anonymous backed
   * pages after unlinking.
   */

  tmpdir = get_tmpdir ();
  template_len = strlen (tmpdir) + 1 + strlen ("sysprof-XXXXXX") + 1;
  template = sysprof_malloc0 (template_len);
  if (template == NULL)
    {
      errno = ENOMEM;
      return -1;
    }

  snprintf (template, template_len, "%s/sysprof-XXXXXX", tmpdir);

  fd = TEMP_FAILURE_RETRY (mkostemp (template, O_BINARY | O_CLOEXEC));
  if (fd < 0)
    {
      free (template);
      return -1;
    }

  unlink (template);
  free (template);

  return fd;
#endif
}

/**
 * sysprof_getpagesize:
 *
 * Gets the page size using platform specific APIs.
 *
 * Returns: the system page size
 *
 * Since: 3.36
 */
size_t
sysprof_getpagesize (void)
{
  return _sysprof_getpagesize ();
}

==> sysprof_48.0.orig.tar.xz: sysprof-48.0/src/libsysprof-capture/sysprof-platform.c <==
/* sysprof-platform.c
 *
 * Copyright 2016-2019 Christian Hergert <chergert@redhat.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * Subject to the terms and conditions of this license, each copyright holder
 * and contributor hereby grants to those receiving rights under this license
 * a perpetual, worldwide, non-exclusive, no-charge, royalty-free,
 * irrevocable (except for failure to satisfy the conditions of this license)
 * patent license to make, have made, use, offer to sell, sell, import, and
 * otherwise transfer this software, where such license applies only to those
 * patent claims, already acquired or hereafter acquired, licensable by such
 * copyright holder or contributor that are necessarily infringed by:
 *
 * (a) their Contribution(s) (the licensed copyrights of copyright holders
 *     and non-copyrightable additions of contributors, in source or binary
 *     form) alone; or
 *
 * (b) combination of their Contribution(s) with the work of authorship to
 *     which such Contribution(s) was added by such copyright holder or
 *     contributor, if, at the time the Contribution is added, such addition
 *     causes such combination to be necessarily infringed. The patent license
 *     shall not apply to any other combinations which include the
 *     Contribution.
 *
 * Except as expressly stated above, no rights or licenses from any copyright
 * holder or contributor is granted under this license, whether expressly, by
 * implication, estoppel or otherwise.
 *
 * DISCLAIMER
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * SPDX-License-Identifier: BSD-2-Clause-Patent
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/syscall.h>
#include <unistd.h>

#include "sysprof-capture-util-private.h"
#include "sysprof-platform.h"

#ifndef __NR_memfd_create
static const char *
get_tmpdir (void)
{
  const char *tmpdir = NULL;

  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = getenv ("TMPDIR");

#ifdef P_tmpdir
  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = P_tmpdir;
#endif

  if (tmpdir == NULL || *tmpdir == '\0')
    tmpdir = "/tmp";

  return tmpdir;
}
#endif  /* !__NR_memfd_create */

/**
 * sysprof_memfd_create:
 * @name: (nullable): A descriptive name for the memfd or %NULL
 *
 * Creates a new memfd using the memfd_create syscall if available.
 * Otherwise, a tmpfile is used. Currently, no attempt is made to
 * ensure the tmpfile is on a tmpfs backed mount.
 *
 * Returns: An fd if successful; otherwise -1 and errno is set.
 */
int
sysprof_memfd_create (const char *name)
{
#ifdef __NR_memfd_create
  if (name == NULL)
    name = "[sysprof]";
  return syscall (__NR_memfd_create, name, 0);
#else
  int fd;
  int flags;
  const char *tmpdir;
  char *template = NULL;
  size_t template_len = 0;

  /*
   * TODO: It would be nice to ensure tmpfs
   *
   * It is not strictly required that the preferred temporary directory
   * will be mounted as tmpfs. We should look through the mounts and ensure
   * that the tmpfile we open is on tmpfs so that we get anonymous backed
   * pages after unlinking.
   */

  tmpdir = get_tmpdir ();
  template_len = strlen (tmpdir) + 1 + strlen ("sysprof-XXXXXX") + 1;
  template = sysprof_malloc0 (template_len);
  if (template == NULL)
    {
      errno = ENOMEM;
      return -1;
    }

  snprintf (template, template_len, "%s/sysprof-XXXXXX", tmpdir);

#ifdef __APPLE__
  flags = 0;
#else
  flags = O_BINARY | O_CLOEXEC;
#endif

  fd = TEMP_FAILURE_RETRY (mkostemp (template, flags));
  if (fd < 0)
    {
      free (template);
      return -1;
    }

  unlink (template);
  free (template);

  return fd;
#endif
}

/**
 * sysprof_getpagesize:
 *
 * Gets the page size using platform specific APIs.
 *
 * Returns: the system page size
 *
 * Since: 3.36
 */
size_t
sysprof_getpagesize (void)
{
  return _sysprof_getpagesize ();
}
